@@ -1,6 +1,6 @@
 /* $RCSfile: extern.h,v $
--- $Revision: 1.8.6.1 $
--- last change: $Author: vq $ $Date: 2005/09/26 00:33:12 $
+-- $Revision: 1.9.12.4 $
+-- last change: $Author: vq $ $Date: 2006/11/12 05:06:38 $
 --
 -- SYNOPSIS
 --      External declarations for dmake functions.
@@ -92,6 +92,38 @@
  * the extern declarations however. */
 #include "posix.h"
 
+
+
+/* Common declarations
+ * ===================
+ * are better made here then in local public.h. So far dmake didn't follow
+ * this strategy but new functions will be added here. */
+
+/* Use our own implementation if no library function is present. */
+#ifndef HAVE_STRLWR
+/* from dmstring.c */
+char *strlwr(char *p);
+#endif
+
+/* from path.c */
+void Clean_path(char *path);
+
+
+
+/* Define some usefull macros. This is done here and not in config.h
+ * to keep this changes usefull even when not using the autotools based
+ * build, i.e. using config.h files that are local to the architecture. */
+#if defined(_WIN32) || defined(__CYGWIN__) || defined(MSDOS) || defined(OS2)
+#  define HAVE_DRIVE_LETTERS 1
+#endif
+
+#if defined(_WIN32) || defined(MSDOS) || defined(OS2) && !defined(__CYGWIN__)
+#  define NULLDEV "NUL"
+#else
+#  define NULLDEV "/dev/null"
+#endif
+
+
 /* Work around some of the functions that may or may not exist */
 #if ! HAVE_TZSET
 #if HAVE_SETTZ
